@@ -420,7 +420,6 @@ gedit_mdi_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 /**
  * gedit_mdi_new:
  * 
@@ -1092,7 +1091,6 @@ gedit_mdi_add_view_handler (BonoboMDI *mdi, GtkWidget *view)
 	return TRUE;
 }
 
-
 static gint 
 gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 {
@@ -1882,7 +1880,6 @@ gedit_window_prefs_save (GeditWindowPrefs *prefs)
 	if ((prefs->statusbar_visible != gedit_prefs_manager_get_statusbar_visible ()) &&
 	    gedit_prefs_manager_statusbar_visible_can_set ())
 		gedit_prefs_manager_set_statusbar_visible (prefs->statusbar_visible);
-
 }
 
 static gchar* 
@@ -1897,7 +1894,7 @@ escape_underscores (const gchar* text)
 
     	length = strlen (text);
 
-	str = g_string_new ("");
+	str = g_string_sized_new (length);
 
   	p = text;
   	end = text + length;
@@ -1935,7 +1932,7 @@ replace_spaces_with_underscores (const gchar* text)
 
     	length = strlen (text);
 
-	str = g_string_new ("");
+	str = g_string_sized_new (length);
 
   	p = text;
   	end = text + length;
@@ -2029,7 +2026,6 @@ language_toggled_handler (
 	gedit_document_set_language (doc, lang);
 }
 
-
 static gchar *
 get_verb_name_for_language (GtkSourceLanguage *lang)
 {
@@ -2206,33 +2202,33 @@ void gedit_mdi_update_languages_menu (BonoboMDI *mdi)
 static void
 update_ui_according_to_state (GeditMDI *mdi)
 {
+	GList *l;
 	GList *views;
 	GdkCursor *cursor;
 	GList *windows;
 
 	gedit_debug (DEBUG_MDI, "");
-	
+
 	/* Upate menus and toolbars */
 	gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
 
 	/* Update views editability */
 	views = bonobo_mdi_get_views (BONOBO_MDI (mdi));
-	while (views != NULL)
+
+	l = views;
+	while (l != NULL)
 	{
 		GeditView *view;
 		GeditDocument *doc;
 
-		view = GEDIT_VIEW (views->data);
+		view = GEDIT_VIEW (l->data);
 		doc = gedit_view_get_document (view);
-		
+
 		switch (gedit_mdi_get_state (mdi))
 		{
 			case GEDIT_STATE_NORMAL:
 				if (!gedit_document_is_readonly (doc))
-				{
 					gedit_view_set_editable (view, TRUE);
-				}
-
 				break;
 
 			case GEDIT_STATE_LOADING:
@@ -2243,7 +2239,7 @@ update_ui_according_to_state (GeditMDI *mdi)
 				break;
 		}
 
-		views = g_list_next (views);
+		l = g_list_next (l);
 	}
 
 	g_list_free (views);
